import socket
import signal
import sys
import logging


from core.config import BankNodeConfig
from utils import setup_logger


class BankNode():
    def __init__(self):
        setup_logger()
        self.logger = logging.getLogger(__name__)
        self._setup_signals()
        self.config = BankNodeConfig()

    def _setup_signals(self):
        self.logger.debug("Setting up exit signal hooks")
        signal.signal(signal.SIGTERM, self.gracefully_exit)
        signal.signal(signal.SIGINT, self.gracefully_exit)

    def start_server(self):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as socket_server:
            socket_server.bind(("127.0.0.1", 6969))
            socket_server.listen()

            while True:
                conn, addr = socket_server.accept()
                with conn:
                    print(f"{addr} connected")
                    request = conn.recv(1024).decode("utf-8")
                    print(f"Request:\n{request}")

    def cleanup(self):
        pass

    def gracefully_exit(self, signum, frame):
        signal_name = signal.Signals(signum).name
        self.logger.warning("Caught %s. Cleaning up before exiting", signal_name)
        self.cleanup()
        sys.exit(0)
